@@ -1,5 +1,6 @@
 # Standard Library
 import http
+import warnings
 from typing import Any, Callable, Dict, List, Optional, Type
 
 # Django
@@ -8,22 +9,36 @@ from django.core.exceptions import ImproperlyConfigured
 from django.db.models import Model
 from django.http import HttpRequest, HttpResponse
 from django.template.engine import Engine
+from django.views.generic.edit import FormMixin
 
 # Local
 from .constants import Action
 from .redirects import HttpResponseSeeOther
-from .response import (
-    TurboFrameResponse,
-    TurboFrameTemplateResponse,
-    TurboStreamResponse,
-    TurboStreamTemplateResponse,
-)
+from .shortcuts import TurboFrame, TurboStream, TurboStreamAction
 
 
-class TurboStreamArgsMixin:
+class TurboStreamMixin:
     turbo_stream_action: Optional[Action] = None
     turbo_stream_target: Optional[str] = None
 
+    def get_turbo_stream(self) -> TurboStreamAction:
+        return TurboStream(self.get_turbo_stream_target_or_raise()).action(
+            self.get_turbo_stream_action_or_raise()
+        )
+
+    def get_turbo_stream_action_or_raise(self) -> Action:
+        """Returns the turbo-stream action parameter
+
+        :return: turbo-stream action
+        :raise: ImproperlyConfigured
+        """
+        action = self.get_turbo_stream_action()
+        if not action:
+            raise ImproperlyConfigured(
+                f"turbo stream action not defined in {self.__class__}.get_turbo_stream_action"
+            )
+        return action
+
     def get_turbo_stream_action(self) -> Optional[Action]:
         """Returns the turbo-stream action parameter
 
@@ -31,6 +46,19 @@ class TurboStreamArgsMixin:
         """
         return self.turbo_stream_action
 
+    def get_turbo_stream_target_or_raise(self) -> str:
+        """Returns the turbo-stream action parameter
+
+        :return: turbo-stream target
+        :raise: ImproperlyConfigured
+        """
+        target = self.get_turbo_stream_target()
+        if not target:
+            raise ImproperlyConfigured(
+                f"turbo stream target not defined in {self.__class__}.get_turbo_stream_target"
+            )
+        return target
+
     def get_turbo_stream_target(self) -> Optional[str]:
         """Returns the turbo-stream target parameter
 
@@ -39,15 +67,27 @@ class TurboStreamArgsMixin:
         return self.turbo_stream_target
 
 
-class TurboFrameArgsMixin:
-    turbo_frame_dom_id = None
+class TurboFrameMixin:
+    turbo_frame_dom_id: Optional[str] = None
+
+    def get_turbo_frame(self) -> TurboFrame:
+        return TurboFrame(self.get_turbo_frame_dom_id_or_raise())
+
+    def get_turbo_frame_dom_id_or_raise(self) -> str:
+
+        dom_id = self.get_turbo_frame_dom_id()
+        if not dom_id:
+            raise ImproperlyConfigured(
+                "turbo frame dom ID not defined in {self.__class__}.get_turbo_frame_dom_id"
+            )
+        return dom_id
 
     def get_turbo_frame_dom_id(self) -> Optional[str]:
         """Should return a valid DOM ID target for the turbo frame."""
         return self.turbo_frame_dom_id
 
 
-class TurboStreamResponseMixin(TurboStreamArgsMixin):
+class TurboStreamResponseMixin(TurboStreamMixin):
     """Handles turbo-stream responses"""
 
     def get_response_content(self) -> str:
@@ -55,26 +95,14 @@ class TurboStreamResponseMixin(TurboStreamArgsMixin):
 
         return ""
 
-    def render_turbo_stream(self, **response_kwargs) -> TurboStreamResponse:
+    def render_turbo_stream(self, **response_kwargs) -> HttpResponse:
         """Returns a turbo-stream response."""
-        target = self.get_turbo_stream_target()
-        action = self.get_turbo_stream_action()
-
-        if target is None:
-            raise ImproperlyConfigured("target is None")
-
-        if action is None:
-            raise ImproperlyConfigured("action is None")
-
-        return TurboStreamResponse(
-            action=action,
-            target=target,
-            content=self.get_response_content(),
-            **response_kwargs,
+        return self.get_turbo_stream().response(
+            self.get_response_content(), **response_kwargs
         )
 
 
-class TurboStreamTemplateResponseMixin(TurboStreamArgsMixin):
+class TurboStreamTemplateResponseMixin(TurboStreamMixin):
     """Handles turbo-stream template responses."""
 
     request: HttpRequest
@@ -83,40 +111,23 @@ class TurboStreamTemplateResponseMixin(TurboStreamArgsMixin):
 
     def render_turbo_stream(
         self, context: Dict[str, Any], **response_kwargs
-    ) -> TurboStreamTemplateResponse:
+    ) -> HttpResponse:
         """Renders a turbo-stream template response.
 
         :param context: template context
         """
 
-        target = self.get_turbo_stream_target()
-        action = self.get_turbo_stream_action()
-
-        if target is None:
-            raise ImproperlyConfigured("target is None")
-
-        if action is None:
-            raise ImproperlyConfigured("action is None")
-
-        return TurboStreamTemplateResponse(
-            request=self.request,
-            template=self.get_template_names(),
-            target=target,
-            action=action,
-            context=context,
-            using=self.template_engine,
+        return (
+            self.get_turbo_stream()
+            .template(self.get_template_names(), context, using=self.template_engine)
+            .response(self.request)
         )
 
 
-class TurboFormMixin:
+class TurboFormMixin(FormMixin):
     """Mixin for handling form validation. Ensures response
     has 422 status on invalid and 303 on success"""
 
-    request: HttpRequest
-    render_to_response: Callable
-    get_success_url: Callable
-    get_context_data: Callable
-
     def form_invalid(self, form: forms.Form) -> HttpResponse:
         return self.render_to_response(
             self.get_context_data(form=form),
@@ -124,6 +135,7 @@ class TurboFormMixin:
         )
 
     def form_valid(self, form: forms.Form) -> HttpResponse:
+        super().form_valid(form)  # type: ignore
         return HttpResponseSeeOther(self.get_success_url())
 
 
@@ -137,7 +149,7 @@ class TurboFormModelMixin(TurboFormMixin):
         return super().form_valid(form)
 
 
-class TurboStreamFormMixin(TurboFormMixin):
+class TurboStreamFormMixin(TurboStreamMixin, TurboFormMixin):
     """Returns a turbo stream when form is invalid.
 
     You should define a partial template corresponding to the template
@@ -162,9 +174,14 @@ class TurboStreamFormMixin(TurboFormMixin):
     partial content will be returned, including any validation errors.
     """
 
-    action: Action = Action.REPLACE
+    # deprecated
     target: Optional[str] = None
 
+    # deprecated
+    action: Action = Action.REPLACE
+
+    turbo_stream_action: Action = Action.REPLACE
+
     turbo_stream_template_prefix: str = "_"
     turbo_stream_template_name: Optional[str] = None
 
@@ -176,14 +193,20 @@ class TurboStreamFormMixin(TurboFormMixin):
         start, join, name = template_name.rpartition("/")
         return start + join + self.turbo_stream_template_prefix + name
 
-    def get_turbo_stream_target(self) -> str:
-        if self.target:
-            return self.target
+    def get_turbo_stream_target(self) -> Optional[str]:
+        # "target" is deprecated: should be turbo_stream_target
+        target = getattr(self, "target", None)
+        if target:
+            warnings.warn("'target' is deprecated: use turbo_stream_target instead")
+            return target
 
-        raise ImproperlyConfigured("target is not provided")
+        return super().get_turbo_stream_target()
 
-    def get_turbo_stream_action(self) -> Action:
-        return self.action
+    def get_turbo_stream_action(self) -> Optional[Action]:
+        if hasattr(self, "action") and self.action != Action.REPLACE:
+            warnings.warn("'action' is deprecated: use turbo_stream_action instead")
+            return self.action
+        return super().get_turbo_stream_action()
 
     def get_turbo_stream_template_names(self) -> List[str]:
         if self.turbo_stream_template_name:
@@ -195,24 +218,21 @@ class TurboStreamFormMixin(TurboFormMixin):
 
     def get_context_data(self, **kwargs) -> Dict[str, Any]:
         if "turbo_stream_target" not in kwargs:
-            kwargs["turbo_stream_target"] = self.get_turbo_stream_target()
+            kwargs["turbo_stream_target"] = self.get_turbo_stream_target_or_raise()
 
         return super().get_context_data(**kwargs)
 
     def render_turbo_stream_response(self, **context) -> HttpResponse:
-        target = self.get_turbo_stream_target()
-        action = self.get_turbo_stream_action()
 
-        return TurboStreamTemplateResponse(
-            request=self.request,
-            template=self.get_turbo_stream_template_names(),
-            target=target,
-            action=action,
+        stream = self.get_turbo_stream()
+
+        return stream.template(
+            self.get_turbo_stream_template_names(),
             context=self.get_context_data(
-                is_turbo_stream=True, target=target, **context
+                is_turbo_stream=True, turbo_stream_target=stream.target, **context
             ),
             using=self.template_engine,
-        )
+        ).response(self.request)
 
     def form_invalid(self, form: forms.Form) -> HttpResponse:
         return self.render_turbo_stream_response(form=form)
@@ -222,46 +242,39 @@ class TurboStreamFormModelMixin(TurboStreamFormMixin):
     object: Optional[Model]
     model: Type[Model]
 
-    def get_turbo_stream_target(self) -> str:
-        try:
-            return super().get_turbo_stream_target()
+    def get_turbo_stream_target(self) -> Optional[str]:
+        target = super().get_turbo_stream_target()
+        if target:
+            return target
 
-        except ImproperlyConfigured:
-            if isinstance(self.object, Model):
-                return f"form-{self.object._meta.model_name}-{self.object.pk}"
+        if isinstance(self.object, Model):
+            return f"form-{self.object._meta.model_name}-{self.object.pk}"
 
-            elif self.model:
-                return f"form-{self.model._meta.model_name}"
+        elif self.model:
+            return f"form-{self.model._meta.model_name}"
 
-            raise
+        return None
 
     def form_valid(self, form: forms.Form) -> HttpResponse:
         self.object = form.save()
         return super().form_valid(form)
 
 
-class TurboFrameResponseMixin(TurboFrameArgsMixin):
+class TurboFrameResponseMixin(TurboFrameMixin):
     """Renders turbo-frame responses"""
 
     def get_response_content(self) -> str:
         return ""
 
-    def render_turbo_frame(self, **response_kwargs) -> TurboFrameResponse:
+    def render_turbo_frame(self, **response_kwargs) -> HttpResponse:
         """Renders a turbo frame to response."""
 
-        dom_id = self.get_turbo_frame_dom_id()
-
-        if dom_id is None:
-            raise ValueError("dom_id must be specified")
-
-        return TurboFrameResponse(
-            content=self.get_response_content(),
-            dom_id=dom_id,
-            **response_kwargs,
+        return self.get_turbo_frame().response(
+            self.get_response_content(), **response_kwargs
         )
 
 
-class TurboFrameTemplateResponseMixin(TurboFrameArgsMixin):
+class TurboFrameTemplateResponseMixin(TurboFrameMixin):
     """Handles turbo-frame template responses."""
 
     request: HttpRequest
@@ -270,20 +283,13 @@ class TurboFrameTemplateResponseMixin(TurboFrameArgsMixin):
 
     def render_turbo_frame(
         self, context: Dict[str, Any], **response_kwargs
-    ) -> TurboFrameTemplateResponse:
+    ) -> HttpResponse:
         """Returns a turbo-frame response.
 
         :param context: template context
         """
-        dom_id = self.get_turbo_frame_dom_id()
-        if dom_id is None:
-            raise ValueError("dom_id must be specified")
-
-        return TurboFrameTemplateResponse(
-            request=self.request,
-            template=self.get_template_names(),
-            dom_id=dom_id,
-            context=context,
-            using=self.template_engine,
-            **response_kwargs,
+        return (
+            self.get_turbo_frame()
+            .template(self.get_template_names(), context, using=self.template_engine)
+            .response(self.request)
         )
